@@ -1,3 +1,20 @@
+# Copyright 2009, 2010 Mark Dickinson.
+#
+# This file is part of the bigfloat module.
+#
+# The bigfloat module is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
+#
+# The bigfloat module is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with the bigfloat module.  If not, see <http://www.gnu.org/licenses/>.
+
 # Python wrapper for MPFR library
 
 from __future__ import with_statement  # for Python 2.5
@@ -385,8 +402,7 @@ def next_up(x, context=None):
                 return y
 
             # otherwise apply mpfr_nextabove
-            bf = Mpfr()
-            mpfr.mpfr_init2(bf, y.precision)
+            bf = Mpfr(y.precision)
             ternary = mpfr.mpfr_set(bf, y._value, 'RoundTiesToEven')
             assert ternary == 0
             mpfr.mpfr_nextabove(bf)
@@ -419,8 +435,7 @@ def next_down(x, context=None):
                 return y
 
             # otherwise apply mpfr_nextabove
-            bf = Mpfr()
-            mpfr.mpfr_init2(bf, y.precision)
+            bf = Mpfr(y.precision)
             ternary = mpfr.mpfr_set(bf, y._value, 'RoundTiesToEven')
             assert ternary == 0
             mpfr.mpfr_nextbelow(bf)
@@ -443,13 +458,12 @@ def _wrap_standard_function(f, argtypes):
             raise TypeError("Wrong number of arguments")
         converted_args = []
         for arg, arg_t in zip(args, argtypes):
-            if arg_t is _pympfr.IMpfr:
+            if arg_t is _pympfr.Mpfr:
                 arg = BigFloat._implicit_convert(arg)._value
             converted_args.append(arg)
         rounding = context.rounding
         converted_args.append(rounding)
-        bf = Mpfr()
-        mpfr.mpfr_init2(bf, context.precision)
+        bf = Mpfr(context.precision)
         ternary = f(bf, *converted_args)
 
         # fit result to current context
@@ -483,7 +497,7 @@ def _wrap_predicate(f):
             raise TypeError("Wrong number of arguments")
         converted_args = []
         for arg, arg_t in zip(args, argtypes):
-            if arg_t is _pympfr.IMpfr:
+            if arg_t is _pympfr.Mpfr:
                 arg = BigFloat._implicit_convert(arg)._value
             converted_args.append(arg)
         return f(*converted_args)
@@ -564,8 +578,7 @@ class BigFloat(object):
         # the testing of that machinery.
 
         # XXX Maybe we should move this function into test_bigfloat
-        bf = Mpfr()
-        mpfr.mpfr_init2(bf, len(value)*4)  # should be sufficient precision
+        bf = Mpfr(len(value)*4) # should be sufficient precision
         ternary = mpfr.mpfr_set_str2(bf, value, 16, 'RoundTiesToEven')
         if ternary:
             # conversion should have been exact, except possibly if
@@ -673,8 +686,7 @@ class BigFloat(object):
 
         """
 
-        m = Mpfr()
-        mpfr.mpfr_init2(m, self.precision)
+        m = Mpfr(self.precision)
         mpfr.mpfr_set(m, self._value, 'RoundTiesToEven')
         if self and is_finite(self):
             mpfr.mpfr_set_exp(m, 0)
@@ -996,7 +1008,6 @@ BigFloat.__sub__ = _binop(sub)
 BigFloat.__mul__ = _binop(mul)
 BigFloat.__div__ = BigFloat.__truediv__ = _binop(div)
 BigFloat.__pow__ = _binop(pow)
-BigFloat.__mod__ = _binop(mod)
 
 # and their reverse operations
 BigFloat.__radd__ = _rbinop(add)
@@ -1004,7 +1015,10 @@ BigFloat.__rsub__ = _rbinop(sub)
 BigFloat.__rmul__ = _rbinop(mul)
 BigFloat.__rdiv__ = BigFloat.__rtruediv__ = _rbinop(div)
 BigFloat.__rpow__ = _rbinop(pow)
-BigFloat.__rmod__ = _rbinop(mod)
+
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+    BigFloat.__mod__ = _binop(mod)
+    BigFloat.__rmod__ = _rbinop(mod)
 
 # comparisons
 BigFloat.__eq__ = _binop(_is_equal)
