@@ -1,15 +1,41 @@
+# Copyright 2009, 2010 Mark Dickinson.
+#
+# This file is part of the bigfloat module.
+#
+# The bigfloat module is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
+#
+# The bigfloat module is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with the bigfloat module.  If not, see <http://www.gnu.org/licenses/>.
+
 from __future__ import with_statement  # for Python 2.5
 
 from contextlib import contextmanager
 import ctypes
 import ctypes.util
+import weakref
+import os.path
+
+try:
+    import bigfloat_config
+except ImportError:
+    bigfloat_config = None
 
-import bigfloat_config
 
 __all__ = [
     # mpfr library, giving access to the various Python-wrapped MPFR functions
     'mpfr',
 
+    # MPFR major and minor version numbers
+    'MPFR_VERSION_MAJOR', 'MPFR_VERSION_MINOR',
+
     # class implementing the floats themselves (corresponds to mpfr_t)
     'Mpfr',
 
@@ -38,11 +64,20 @@ __all__ = [
 ################################################################################
 # Locate and load the library
 
-_locate_error_message = \
-"""Failed to find the MPFR library.  Please ensure that the MPFR
-library is installed on your system, and if necessary edit the
-'mpfr_library_location' entry in the bigfloat_config.py configuration
-file to give the location of the library."""
+_locate_error_message = """\
+Failed to find the MPFR library.  Please ensure that the MPFR library
+is installed on your system and is in the library search path.  You
+may need to set LD_LIBRARY_PATH (or DYLD_LIBRARY_PATH on OS X) for
+this.
+
+If necessary you can edit the 'mpfr_library_location' entry in the
+bigfloat_config.py configuration file to give the location of the
+library.  This configuration file can be found in the same place that
+the bigfloat library was installed:
+
+%s
+""" % (os.path.join(
+        os.path.abspath(os.path.dirname(__file__)), 'bigfloat_config.py'),)
 
 # try the config file first
 _mpfr_library_name = getattr(bigfloat_config, 'mpfr_library_location', None)
@@ -54,6 +89,18 @@ if not _mpfr_library_name:
 
 mpfr = ctypes.cdll.LoadLibrary(_mpfr_library_name)
 
+# get library version, and check that it's new enough
+mpfr.mpfr_get_version.argtypes = []
+mpfr.mpfr_get_version.restype = ctypes.c_char_p
+
+_mpfr_version = mpfr.mpfr_get_version()
+MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR, _ = _mpfr_version.split('.', 2)
+MPFR_VERSION_MAJOR = int(MPFR_VERSION_MAJOR)
+MPFR_VERSION_MINOR = int(MPFR_VERSION_MINOR)
+
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) < (2, 3):
+    raise RuntimeError("This module requires MPFR version 2.3 or later; "
+                       "found version %s" % _mpfr_version)
 
 ################################################################################
 # Platform dependent values
@@ -162,7 +209,51 @@ class __mpfr_struct(ctypes.Structure):
         ("_mpfr_d", ctypes.POINTER(mpfr_limb_t))
         ]
 
-Mpfr = __mpfr_struct * 1
+PreMpfr = __mpfr_struct * 1
+
+# Wrapper for the PreMpfr type that ensures initialization, and
+# automatic calling of mpfr.mpfr_clear when an Mpfr instance goes out
+# of scope.
+
+# We ensure mpfr.mpfr_clear gets called by using a suitable weakref
+# callback.
+
+class Mpfr(object):
+    # Dictionary from weakrefs to PreMpfr instances.
+    _values = {}
+    def __new__(cls, precision):
+        self = object.__new__(cls)
+        self._value = PreMpfr()
+        wref = weakref.ref(self, self._clear_value)
+        self._values[wref] = self._value
+        self._initialized = False
+        return self
+
+    @classmethod
+    def _clear_value(cls, wref):
+        mpfr.mpfr_clear(cls._values[wref])
+        del cls._values[wref]
+
+    def __init__(self, precision):
+        if self._initialized:
+            mpfr.mpfr_set_prec(self._value, precision)
+        else:
+            mpfr.mpfr_init2(self._value, precision)
+            self._initialized = True
+
+    @classmethod
+    def from_param(cls, self):
+        return self._value
+
+    def __repr__(self):
+        # Crude but usable string representation for Mpfr type.
+        negative, digits, exp = get_str2(self, 10, 0, RoundTiesToEven)
+        if '@NaN@' not in digits and '@Inf@' not in digits:
+            digits = '0.' + digits + 'E' + str(exp)
+        return ('-' if negative else '') + digits
+
+    __str__ = __repr__
+
 
 # Precision class used for automatic range checking of precisions.
 # Arguments of type mpfr_prec_t should use the Precision class.
@@ -237,49 +328,9 @@ def Ternary(x):
     else:
         return 0
 
-# Type UMpfr represents an uninitialized Mpfr instance
-
-class UMpfr(object):
-    @classmethod
-    def from_param(cls, value):
-        if not isinstance(value, Mpfr):
-            raise TypeError("Expecting argument of type Mpfr")
-        if hasattr(value, '_initialized'):
-            raise ValueError("Mpfr instance is already initialized")
-        return value
-
-# Type IMpfr represents an initialized Mpfr instance
-
-class IMpfr(object):
-    @classmethod
-    def from_param(cls, value, _Mpfr=Mpfr):
-        # this function can end up being called (via Mpfr.__del__) at
-        # interpreter shutdown time, possibly after the module global
-        # 'Mpfr' has been deleted.  Hence the _Mpfr hack.
-        if not isinstance(value, _Mpfr):
-            raise TypeError("Expecting argument of type Mpfr")
-        if not hasattr(value, '_initialized'):
-            raise ValueError("Mpfr instance is not initialized")
-        return value
-
 ################################################################################
 # Various useful errcheck functions
 
-def set_init(result, func, args):
-    mpfr_instance = args[0]
-    assert not hasattr(mpfr_instance, '_initialized')
-    mpfr_instance._initialized=True
-
-def clear_init(result, func, args):
-    mpfr_instance = args[0]
-    assert hasattr(mpfr_instance, '_initialized')
-    del mpfr_instance._initialized
-
-def set_init_on_success(result, func, args):
-    if result:
-        raise ValueError("Initialization call failed")
-    set_init(result, func, args)
-
 def error_on_failure(result, func, args):
     if result:
         raise ValueError("call failed")
@@ -306,22 +357,15 @@ MPFR_EMIN_DEFAULT = -MPFR_EMAX_DEFAULT
 
 # 5.1 Initialization Functions
 
-# Only the initialization functions take arguments of type UMpfr.  All
-# other functions use type IMpfr instead.  The reason is that the IMpfr
-# type implements a check that the underlying Mpfr instance is
-# already initialized, but mpfr_init and mpfr_init2 expect to receive
-# uninitialized instances.
-
 mpfr_functions = [
 
     # 5.1: Initialization Functions
-    ('init2', [UMpfr, Precision], None, set_init),
-    ('clear', [IMpfr], None, clear_init),
-    ('init', [UMpfr], None, set_init),
+    ('init2', [PreMpfr, Precision], None),
+    ('clear', [PreMpfr], None),
     ('set_default_prec', [Precision], None),
     ('get_default_prec', [], mpfr_prec_t),
-    ('set_prec', [IMpfr, Precision], None),
-    ('get_prec', [IMpfr], mpfr_prec_t),
+    ('set_prec', [PreMpfr, Precision], None),
+    ('get_prec', [Mpfr], mpfr_prec_t),
 
     # 5.2 Assignment Functions
 
@@ -335,60 +379,55 @@ mpfr_functions = [
 
     # use of mpfr_set_str is not recommended, since it doesn't return
     # the ternary value.  Use set_str2 (defined below) instead.
-    ('set_str', [IMpfr, ctypes.c_char_p, Base, RoundingMode], ctypes.c_int,
+    ('set_str', [Mpfr, ctypes.c_char_p, Base, RoundingMode], ctypes.c_int,
      error_on_failure),
-    ('strtofr', [IMpfr, ctypes.c_char_p, ctypes.POINTER(ctypes.c_char_p), 
+    ('strtofr', [Mpfr, ctypes.c_char_p, ctypes.POINTER(ctypes.c_char_p),
                  Base, RoundingMode], Ternary),
-    ('set_inf', [IMpfr, Sign], None),
-    ('set_nan', [IMpfr], None),
-    ('swap', [IMpfr, IMpfr], None),
-
-    # 5.3 Combined Initialization and Assignment Functions
-    # Apart from init_set_str, these are all macros
-    ('init_set_str', [UMpfr, ctypes.c_char_p, Base, RoundingMode],
-     ctypes.c_int, set_init_on_success),
+    ('set_inf', [Mpfr, Sign], None),
+    ('set_nan', [Mpfr], None),
+    ('swap', [Mpfr, Mpfr], None),
 
     # 5.4 Conversion Functions
-    ('get_d', [IMpfr, RoundingMode], ctypes.c_double),
-    ('get_d_2exp', [ctypes.POINTER(ctypes.c_long), IMpfr, RoundingMode],
+    ('get_d', [Mpfr, RoundingMode], ctypes.c_double),
+    ('get_d_2exp', [ctypes.POINTER(ctypes.c_long), Mpfr, RoundingMode],
      ctypes.c_double),
-    ('get_si', [IMpfr, RoundingMode], ctypes.c_long),
-    ('get_ui', [IMpfr, RoundingMode], ctypes.c_ulong),
+    ('get_si', [Mpfr, RoundingMode], ctypes.c_long),
+    ('get_ui', [Mpfr, RoundingMode], ctypes.c_ulong),
     ('get_str', [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(mpfr_exp_t),
-                 Base, Size_t, IMpfr, RoundingMode],
+                 Base, Size_t, Mpfr, RoundingMode],
      ctypes.POINTER(ctypes.c_char)),
     ('free_str', [ctypes.POINTER(ctypes.c_char)], None),
-    ('fits_ulong_p', [IMpfr, RoundingMode], bool),
-    ('fits_slong_p', [IMpfr, RoundingMode], bool),
-    ('fits_uint_p', [IMpfr, RoundingMode], bool),
-    ('fits_sint_p', [IMpfr, RoundingMode], bool),
-    ('fits_ushort_p', [IMpfr, RoundingMode], bool),
-    ('fits_sshort_p', [IMpfr, RoundingMode], bool),
+    ('fits_ulong_p', [Mpfr, RoundingMode], bool),
+    ('fits_slong_p', [Mpfr, RoundingMode], bool),
+    ('fits_uint_p', [Mpfr, RoundingMode], bool),
+    ('fits_sint_p', [Mpfr, RoundingMode], bool),
+    ('fits_ushort_p', [Mpfr, RoundingMode], bool),
+    ('fits_sshort_p', [Mpfr, RoundingMode], bool),
 
     # 5.5 Basic Arithmetic Functions
     # All of these are standard functions, declared below.
     # Functions involved GMP mpz_t and mpq_t types aren't wrapped.
 
     # 5.6 Comparison Functions
-    ('cmp', [IMpfr, IMpfr], Ternary),
-    ('cmp_ui', [IMpfr, UnsignedLong], Ternary),
-    ('cmp_si', [IMpfr, Long], Ternary),
-    ('cmp_d', [IMpfr, ctypes.c_double], Ternary),
-    ('cmp_ui_2exp', [IMpfr, UnsignedLong, Exponent], Ternary),
-    ('cmp_si_2exp', [IMpfr, Long, Exponent], Ternary),
-    ('cmpabs', [IMpfr, IMpfr], Ternary),
-    
+    ('cmp', [Mpfr, Mpfr], Ternary),
+    ('cmp_ui', [Mpfr, UnsignedLong], Ternary),
+    ('cmp_si', [Mpfr, Long], Ternary),
+    ('cmp_d', [Mpfr, ctypes.c_double], Ternary),
+    ('cmp_ui_2exp', [Mpfr, UnsignedLong, Exponent], Ternary),
+    ('cmp_si_2exp', [Mpfr, Long, Exponent], Ternary),
+    ('cmpabs', [Mpfr, Mpfr], Ternary),
+
     # nan_p, inf_p, number_p, zero_p are standard predicates, declared below
-    ('sgn', [IMpfr], Ternary),
+    ('sgn', [Mpfr], Ternary),
 
     # remaining functions are again standard predicates, declared below
 
     # 5.7 Special Functions
 
     # Most of these are standard functions, declared below.
-    ('sin_cos', [IMpfr, IMpfr, IMpfr, RoundingMode], Ternary),
-    ('sinh_cosh', [IMpfr, IMpfr, IMpfr, RoundingMode], Ternary),
-    ('lgamma', [IMpfr, ctypes.POINTER(ctypes.c_int), IMpfr, RoundingMode], Ternary),
+    ('sin_cos', [Mpfr, Mpfr, Mpfr, RoundingMode], Ternary),
+    ('lgamma', [Mpfr, ctypes.POINTER(ctypes.c_int), Mpfr, RoundingMode],
+     Ternary),
     ('free_cache', [], None),
 
     # we don't wrap mpfr_sum at the moment
@@ -401,36 +440,35 @@ mpfr_functions = [
     # note that 'rint' is *not* a standard function: the rounding
     # mode determines the actual operation, not the method of
     # rounding the result.
-    ('rint', [IMpfr, IMpfr, RoundingMode], Ternary),
-    ('ceil', [IMpfr, IMpfr], Ternary),
-    ('floor', [IMpfr, IMpfr], Ternary),
-    ('round', [IMpfr, IMpfr], Ternary),
-    ('trunc', [IMpfr, IMpfr], Ternary),
+    ('rint', [Mpfr, Mpfr, RoundingMode], Ternary),
+    ('ceil', [Mpfr, Mpfr], Ternary),
+    ('floor', [Mpfr, Mpfr], Ternary),
+    ('round', [Mpfr, Mpfr], Ternary),
+    ('trunc', [Mpfr, Mpfr], Ternary),
 
     # rint_ceil, rint_floor, rint_round, rint_trunc are standard
     # frac is standard
-    ('modf', [IMpfr, IMpfr, IMpfr, RoundingMode], Ternary),
     # fmod, remainder are standard
-    ('remquo', [IMpfr, ctypes.POINTER(ctypes.c_long), IMpfr, IMpfr, RoundingMode],
+    ('remquo', [Mpfr, ctypes.POINTER(ctypes.c_long), Mpfr, Mpfr, RoundingMode],
      Ternary),
     # integer_p is a standard predicate
 
     # 5.11 Rounding Related Functions
     ('set_default_rounding_mode', [RoundingMode], None),
     ('get_default_rounding_mode', [], mpfr_rnd_t, convert_to_rounding_mode),
-    ('prec_round', [IMpfr, Precision, RoundingMode], Ternary),
-    ('can_round', [IMpfr, Exponent, RoundingMode, RoundingMode, Precision],
+    ('prec_round', [Mpfr, Precision, RoundingMode], Ternary),
+    ('can_round', [Mpfr, Exponent, RoundingMode, RoundingMode, Precision],
      bool),
     ('print_rnd_mode', [RoundingMode], ctypes.c_char_p),
 
     # 5.12 Miscellaneous Functions
-    ('nexttoward', [IMpfr, IMpfr], None),
-    ('nextabove', [IMpfr], None),
-    ('nextbelow', [IMpfr], None),
+    ('nexttoward', [Mpfr, Mpfr], None),
+    ('nextabove', [Mpfr], None),
+    ('nextbelow', [Mpfr], None),
     # min and max are standard functions; see below
     # urandomb not supported
-    ('get_exp', [IMpfr], mpfr_exp_t),
-    ('set_exp', [IMpfr, Exponent], ctypes.c_int, error_on_failure),
+    ('get_exp', [Mpfr], mpfr_exp_t),
+    ('set_exp', [Mpfr, Exponent], ctypes.c_int, error_on_failure),
     # signbit is a standard predicate
     # setsign and copysign are standard functions
     ('get_version', [], ctypes.c_char_p),
@@ -445,8 +483,8 @@ mpfr_functions = [
     ('get_emin_max', [], mpfr_exp_t),
     ('get_emax_min', [], mpfr_exp_t),
     ('get_emax_max', [], mpfr_exp_t),
-    ('check_range', [IMpfr, ctypes.c_int, RoundingMode], Ternary),
-    ('subnormalize', [IMpfr, ctypes.c_int, RoundingMode], Ternary),
+    ('check_range', [Mpfr, ctypes.c_int, RoundingMode], Ternary),
+    ('subnormalize', [Mpfr, ctypes.c_int, RoundingMode], Ternary),
     # for flag set, clear and test functions see below
     ('clear_flags', [], None),
 
@@ -455,6 +493,13 @@ mpfr_functions = [
     # Functions from 5.14 and 5.15 not wrapped.
     ]
 
+# functions that are new in MPFR version 2.4
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+    mpfr_functions.extend([
+            ('modf', [Mpfr, Mpfr, Mpfr, RoundingMode], Ternary),
+            ('sinh_cosh', [Mpfr, Mpfr, Mpfr, RoundingMode], Ternary),
+            ])
+
 # additional functions for flags in section 13
 mpfr_flags = ['underflow', 'overflow', 'nanflag', 'inexflag', 'erangeflag']
 for f in mpfr_flags:
@@ -489,7 +534,7 @@ standard_constants = [
 
 standard_unary_functions = [
     'set', 'neg', 'abs',
-    'sqr', 'sqrt', 'rec_sqrt', 'cbrt',
+    'sqr', 'sqrt', 'cbrt',
     'log', 'log2', 'log10',
     'exp', 'exp2', 'exp10',
     'cos', 'sin', 'tan',
@@ -499,7 +544,7 @@ standard_unary_functions = [
     'sech', 'csch', 'coth',
     'acosh', 'asinh', 'atanh',
     'log1p', 'expm1',
-    'eint', 'li2',
+    'eint',
     'gamma', 'lngamma',
     'zeta',
     'erf', 'erfc',
@@ -508,16 +553,28 @@ standard_unary_functions = [
     'frac',
     ]
 
+# functions that are new in MPFR version 2.4
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+    standard_unary_functions.extend([
+            'li2', 'rec_sqrt',
+            ])
+
 # standard functions taking two Mpfr instances as input
 
 standard_binary_functions = [
     'add', 'sub', 'mul', 'div', 'pow',
     'dim', 'atan2', 'agm', 'hypot',
-    'fmod', 'remainder',
+    'remainder',
     'max', 'min',
     'copysign',
     ]
 
+# functions that are new in MPFR version 2.4
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+    standard_binary_functions.extend([
+            'fmod',
+            ])
+
 # and three Mpfr instances...
 
 standard_ternary_functions = [
@@ -534,50 +591,55 @@ additional_standard_functions = [
     ('set_si_2exp', [Long, Exponent]),
 
     # 5.5 Basic Arithmetic Functions
-    ('add_ui', [IMpfr, UnsignedLong]),
-    ('add_si', [IMpfr, Long]),
-    ('add_d', [IMpfr, ctypes.c_double]),
-    ('ui_sub', [UnsignedLong, IMpfr]),
-    ('sub_ui', [IMpfr, UnsignedLong]),
-    ('si_sub', [Long, IMpfr]),
-    ('sub_si', [IMpfr, Long]),
-    ('d_sub', [ctypes.c_double, IMpfr]),
-    ('sub_d', [IMpfr, ctypes.c_double]),
-    ('mul_ui', [IMpfr, UnsignedLong]),
-    ('mul_si', [IMpfr, Long]),
-    ('mul_d', [IMpfr, ctypes.c_double]),
-    ('ui_div', [UnsignedLong, IMpfr]),
-    ('div_ui', [IMpfr, UnsignedLong]),
-    ('si_div', [Long, IMpfr]),
-    ('div_si', [IMpfr, Long]),
-    ('d_div', [ctypes.c_double, IMpfr]),
-    ('div_d', [IMpfr, ctypes.c_double]),
+    ('add_ui', [Mpfr, UnsignedLong]),
+    ('add_si', [Mpfr, Long]),
+    ('ui_sub', [UnsignedLong, Mpfr]),
+    ('sub_ui', [Mpfr, UnsignedLong]),
+    ('si_sub', [Long, Mpfr]),
+    ('sub_si', [Mpfr, Long]),
+    ('mul_ui', [Mpfr, UnsignedLong]),
+    ('mul_si', [Mpfr, Long]),
+    ('ui_div', [UnsignedLong, Mpfr]),
+    ('div_ui', [Mpfr, UnsignedLong]),
+    ('si_div', [Long, Mpfr]),
+    ('div_si', [Mpfr, Long]),
     ('sqrt_ui', [UnsignedLong]),
-    ('root', [IMpfr, UnsignedLong]),
-    ('pow_ui', [IMpfr, UnsignedLong]),
-    ('pow_si', [IMpfr, Long]),
+    ('root', [Mpfr, UnsignedLong]),
+    ('pow_ui', [Mpfr, UnsignedLong]),
+    ('pow_si', [Mpfr, Long]),
     ('ui_pow_ui', [UnsignedLong, UnsignedLong]),
-    ('ui_pow', [UnsignedLong, IMpfr]),
-    ('mul_2ui', [IMpfr, UnsignedLong]),
-    ('mul_2si', [IMpfr, Long]),
-    ('div_2ui', [IMpfr, UnsignedLong]),
-    ('div_2si', [IMpfr, Long]),
+    ('ui_pow', [UnsignedLong, Mpfr]),
+    ('mul_2ui', [Mpfr, UnsignedLong]),
+    ('mul_2si', [Mpfr, Long]),
+    ('div_2ui', [Mpfr, UnsignedLong]),
+    ('div_2si', [Mpfr, Long]),
 
     # 5.7 Special Functions
     ('fac_ui', [UnsignedLong]),
     ('zeta_ui', [UnsignedLong]),
-    ('jn', [Long, IMpfr]),
-    ('yn', [Long, IMpfr]),
+    ('jn', [Long, Mpfr]),
+    ('yn', [Long, Mpfr]),
 
     # 5.12 Miscellaneous Functions
-    ('setsign', [IMpfr, Bool]),
+    ('setsign', [Mpfr, Bool]),
 ]
 
+# functions that are new in MPFR version 2.4
+if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+    additional_standard_functions.extend([
+            ('add_d', [Mpfr, ctypes.c_double]),
+            ('sub_d', [Mpfr, ctypes.c_double]),
+            ('d_sub', [ctypes.c_double, Mpfr]),
+            ('mul_d', [Mpfr, ctypes.c_double]),
+            ('div_d', [Mpfr, ctypes.c_double]),
+            ('d_div', [ctypes.c_double, Mpfr]),
+            ])
+
 standard_functions = \
     [(name, []) for name in standard_constants] + \
-    [(name, [IMpfr]) for name in standard_unary_functions] + \
-    [(name, [IMpfr]*2) for name in standard_binary_functions] + \
-    [(name, [IMpfr]*3) for name in standard_ternary_functions] + \
+    [(name, [Mpfr]) for name in standard_unary_functions] + \
+    [(name, [Mpfr]*2) for name in standard_binary_functions] + \
+    [(name, [Mpfr]*3) for name in standard_ternary_functions] + \
     additional_standard_functions
 
 # Additional standard functions that aren't in MPFR itself, but are
@@ -585,7 +647,7 @@ standard_functions = \
 
 extra_standard_functions = [
     ('set_str2', [ctypes.c_char_p, Base]),
-    ('lgamma_nosign', [IMpfr]),
+    ('lgamma_nosign', [Mpfr]),
     ]
 
 unary_predicates = [
@@ -600,11 +662,11 @@ binary_predicates = [
     ]
 
 predicates = \
-    [(name, [IMpfr]) for name in unary_predicates] + \
-    [(name, [IMpfr]*2) for name in binary_predicates]
+    [(name, [Mpfr]) for name in unary_predicates] + \
+    [(name, [Mpfr]*2) for name in binary_predicates]
 
 for fn, args in standard_functions:
-    mpfr_functions.append((fn, [IMpfr] + args + [RoundingMode], Ternary))
+    mpfr_functions.append((fn, [Mpfr] + args + [RoundingMode], Ternary))
 
 for pred, args in predicates:
     mpfr_functions.append((pred, args, bool))
@@ -722,28 +784,6 @@ def remquo2(r, x, y, rnd):
     ternary = mpfr.mpfr_remquo(r, ctypes.byref(q), x, y, rnd)
     return ternary, q
 
-################################################################################
-# A couple of extra niceties to make the Mpfr class easier to use.
-
-# First, we monkeypatch the Mpfr class to give it a crude but usable
-# string representation.
-
-def _mpfr_str(self):
-    negative, digits, exp = get_str2(self, 10, 0, RoundTiesToEven)
-    if '@NaN@' not in digits and '@Inf@' not in digits:
-        digits = '0.' + digits + 'E' + str(exp)
-    return ('-' if negative else '') + digits
-
-Mpfr.__repr__ = Mpfr.__str__ = _mpfr_str
-
-# Add a __del__ method to ensure that Mpfr instances are automatically
-# cleared before they're garbage collected.
-
-def _mpfr_del(self, _clear_fn = mpfr.mpfr_clear):
-    if hasattr(self, '_initialized'):
-        _clear_fn(self)
-
-Mpfr.__del__ = _mpfr_del
 
 ################################################################################
 # Context manager to give an easy way to change emin and emax temporarily.
