@@ -1,3 +1,20 @@
+# Copyright 2009, 2010 Mark Dickinson.
+#
+# This file is part of the bigfloat module.
+#
+# The bigfloat module is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
+#
+# The bigfloat module is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with the bigfloat module.  If not, see <http://www.gnu.org/licenses/>.
+
 from __future__ import with_statement # for Python 2.5
 
 from test.test_support import run_unittest
@@ -5,7 +22,9 @@ import unittest
 import operator
 from bigfloat import *
 from bigfloat import _all_flags
+from bigfloat import MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR
 import math
+import sys
 import __builtin__
 
 all_rounding_modes = [RoundTowardZero, RoundTowardNegative,
@@ -107,7 +126,11 @@ class BigFloatTests(unittest.TestCase):
     def test_arithmetic_functions(self):
         # test add, mul, div, sub, pow, mod
         test_precisions = [2, 10, 23, 24, 52, 53, 54, 100]
-        fns = [add, sub, mul, div, pow, mod]
+        fns = [add, sub, mul, div, pow]
+        # mod function only exists for MPFR version >= 2.4.0
+        if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+            fns.append(mod)
+
         values = [2, 3L, 1.234, BigFloat('0.678'), BigFloat('nan'),
                   float('0.0'), float('inf'), True]
 
@@ -152,8 +175,12 @@ class BigFloatTests(unittest.TestCase):
         # whether or not 'from __future__ import division' is enabled.
         # So we test both operator.div and operator.truediv.
         operations = [operator.add, operator.mul, operator.div,
-                      operator.sub, operator.pow, operator.truediv,
-                      operator.mod]
+                      operator.sub, operator.pow, operator.truediv]
+
+        # % operator only works for MPFR version >= 2.4.
+        if (MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR) >= (2, 4):
+            operations.append(operator.mod)
+
         for value in other_values:
             for p in test_precisions:
                 with precision(p):
@@ -540,16 +567,6 @@ class BigFloatTests(unittest.TestCase):
         self.assertEqual(hash(x1), hash(x2))
         self.assertEqual(hash(x1), hash(x3))
 
-        # check that hash(n) matches hash(BigFloat(n)) for integers n
-        for n in range(-50, 50):
-            self.assertEqual(hash(n), hash(BigFloat.exact(n)))
-
-        #   values near powers of 2
-        for e in [30, 31, 32, 33, 34, 62, 63, 64, 65, 66]:
-            for n in range(2**e-50, 2**e+50):
-                self.assertEqual(hash(n), hash(BigFloat.exact(n)))
-                self.assertEqual(hash(BigFloat(n)), hash(int(BigFloat(n))))
-
         # check that hash values match those of floats
         self.assertEqual(hash(BigFloat('inf')), hash(float('inf')))
         self.assertEqual(hash(BigFloat('-inf')), hash(float('-inf')))
@@ -560,6 +577,28 @@ class BigFloatTests(unittest.TestCase):
         self.assertEqual(hash(BigFloat('1.625')), hash(float('1.625')))
         self.assertEqual(hash(BigFloat.exact(1.1)), hash(1.1))
 
+        # check that hash(n) matches hash(BigFloat(n)) for integers n
+        for n in range(-50, 50):
+            self.assertEqual(hash(n), hash(BigFloat.exact(n)))
+
+        # XXX It's expected that these tests will fail on Python 2.5,
+        # due to the unpredictability of the integer hashing algorithm.
+        # There are no plans to fix this problem (and it's not really
+        # a problem unless you're putting both integers and BigFloat
+        # instances into the same set or dict).
+        if sys.version_info < (2, 6):
+            return
+
+        # values near powers of 2
+        for e in [30, 31, 32, 33, 34, 62, 63, 64, 65, 66]:
+            for n in range(2**e-50, 2**e+50):
+                self.assertEqual(hash(n), hash(BigFloat.exact(n)),
+                                 "hash(n) != hash(BigFloat.exact(n)) "
+                                 "for n = %s" % n)
+                self.assertEqual(hash(BigFloat(n)), hash(int(BigFloat(n))),
+                                 "hash(BigFloat(n)) != hash(int(BigFloat(n))) "
+                                 "for n = %s" % n)
+
     def test_hex(self):
         # test conversion to a hex string
 
